// (A) FILES TO CACHE
const cName = "v1",
cList = ["img-science.png", "img-not-found.png"];

// (B) ADD FILES INTO CACHE ON WORKER INSTALL
self.addEventListener("install", (evt) => {
  evt.waitUntil(
    caches.open(cName)
    .then((cache) => { return cache.addAll(cList); })
    .catch((err) => { console.log(err); })
  );
  console.log("Files cached");
});

// (C) "HIJACK" FETCH REQUESTS
self.addEventListener("fetch", (evt) => {
  evt.respondWith(
    caches.match(evt.request).then((res) => {
      // (C1) FILE FOUND IN CACHE - SERVE CACHED COPY
      if (res !== undefined) { return res; }

      // (C2) ELSE PROCEED WITH NETWORK LOAD
      else {
        return fetch(evt.request).then((res) => {
          // REQUESTED FILE TYPE
          let type = res.url.split(/[#?]/)[0].split('.').pop().trim().toLowerCase(),
              imgs = ["png", "jpg", "jpeg", "webp", "gif"];

          // CACHE IMAGES ONLY
          // RES CAN ONLY BE USED ONCE, THUS THE CLONE()
          if (res.status==200 && imgs.includes(type)) {
            let clone = res.clone();
            caches.open("v1").then((cache) => {
              cache.put(evt.request, clone);
            });
          }

          // NOT FOUND - SERVE STANDARD "NOT FOUND IMAGE"
          if (res.status==404 && imgs.includes(type)) {
            return caches.match("/img-not-found.png");
          }

          // RETURN WHATEVER SERVER RETURNS
          return res;
        });
      }
    })
  );
});
