var chat = {
  // (A) PROPERTIES
  // (A1) CHAT SERVER
  peer : null, conn : null,
  opt : { host: "localhost", port: 9000, path: "/myapp" },
  // (A2) HTML ELEMENTS
  hView : null, hForm : null, hTxt : null, hGo : null, hCx : null,

  // (B) INIT
  init : (id, after) => {
    // (B1) GET + SET HTML ELEMENTS
    chat.hView = document.getElementById("chatView");
    chat.hForm = document.getElementById("chatForm");
    chat.hTxt = document.getElementById("chatTxt");
    chat.hGo = document.getElementById("chatGo");
    chat.hCx = document.getElementById("chatCx");
    chat.hForm.onsubmit = chat.send;
    chat.hCx.onclick = chat.disconnect;

    // (B2) HANDSHAKE WITH PEER SERVER
    chat.peer = new Peer(id, chat.opt);

    // (B3) READY
    chat.peer.on("open", (id) => {
      if (after) { after(); }
    });

    // (B4) ON PEER CONNECT
    chat.peer.on("connection", (conn) => {
      chat.conn = conn;
      chat.connector();
    });

    // (B5) ALL OTHER PEER EVENTS
    chat.peer.on("close", () => { console.log("close"); });
    chat.peer.on("disconnected", () => { console.log("dc"); });
    chat.peer.on("error", (err) => { console.log(err); });
    // chat.peer.on("call", () => { console.log("call"); });
  },

  // (C) CONNECT TO PEER
  connect : (id) => {
    chat.conn = chat.peer.connect(id);
    chat.connector();
  },

  // (D) ATTACH PEER LISTENERS
  connector : () => {
    chat.conn.on("open", () => { chat.tog(); });
    chat.conn.on("close", () => { chat.tog(true); });
    chat.conn.on("data", chat.recv);
    chat.conn.on("error", (err) => { console.log(err); });
  },

  // (D) DISCONNECT PEER
  disconnect : () => { chat.conn.close(); },

  // (E) HELPER - DRAW MESSAGE
  draw : (txt, css) => {
    let row = document.createElement("div");
    row.innerHTML = txt;
    row.className = css;
    chat.hView.appendChild(row);
  },

  // (F) SEND MESSAGE
  send : () => {
    chat.conn.send(chat.hTxt.value);
    chat.draw(chat.hTxt.value, "send");
    chat.hTxt.value = "";
    return false;
  },

  // (G) RECEIVE MESSAGE
  recv : (txt) => {
    chat.draw(txt, "recv");
  },

  // (H) TOGGLE INTERFACE
  tog : (lock) => {
    if (lock) {
      chat.hForm.disabled = true;
      chat.hTxt.disabled = true;
      chat.hGo.disabled = true;
      chat.hCx.disabled = true;
    } else {
      chat.hForm.disabled = false;
      chat.hTxt.disabled = false;
      chat.hGo.disabled = false;
      chat.hCx.disabled = false;
    }
  }
};
