// CREDITS : https://dev.to/abdisalan_js/how-to-code-a-video-streaming-server-using-nodejs-2o0
// (A) MODULES & EXPRESS SERVER
const express = require("express"),
      fs = require("fs"),
      app = express();

// (B) SERVE DEFAULT HTML PAGE
app.get("/", (req, res) => {
  res.sendFile(__dirname + "/6a-vid-page.html");
});

// (C) SERVE VIDEO STREAM
app.get("/video", (req, res) => {
  // (C1) VIDEO RANGE MUST BE SPECIFIED
  const range = req.headers.range;
  if (!range) {
    res.status(400).send("Requires range header");
  }

  // (C2) VIDEO STATS
  const vid = __dirname + "/video.mp4", // CHANGE TO YOUR OWN!
        vidSize = fs.statSync(vid).size,
        chunkSize = 10 ** 6, // 1MB
        start = Number(range.replace(/\D/g, "")),
        end = Math.min(start + chunkSize, vidSize - 1),
        contentLength = end - start + 1;

  // (C3) SERVE HTTP 206 (PARTIAL CONTENT)
  res.writeHead(206, {
    "Content-Range": `bytes ${start}-${end}/${vidSize}`,
    "Accept-Ranges": "bytes",
    "Content-Length": contentLength,
    "Content-Type": "video/mp4",
  });

  // (C4) SERVE VIDEO STREAM
  fs.createReadStream(vid, { start, end }).pipe(res);
});

// (D) LAUNCH!
app.listen(8080, () => {
  console.log("Server deployed at 8080!");
});
